import { ErrorDetailMap, ErrorMap } from '@genesislcap/foundation-utils';
import { BindingObserver, ExecutionContext } from '@microsoft/fast-element';
import { Container } from '@microsoft/fast-foundation';
import type { AnyEventObject, EventObject, InternalMachineImplementations, InterpreterFrom, InterpreterOptions, MachineContext as XMachineContext, NoInfer, ParameterizedObject, ProvidedActor, ResolveTypegenMeta, State, StateMachine, StateValue, StateValueFrom, TypegenDisabled } from 'xstate';
import { SubscriberChangeCallback } from './binding';
/**
 * MachineContext interface.
 *
 * @remarks
 * The interface of our xstate based machine contexts that include `error` and `errors` properties. Should be extended.
 *
 * @example Creating a machine context.
 * ```ts
 * export interface MyMachineContext extends MachineContext {
 *   ...
 * };
 * ```
 *
 * @public
 */
export interface MachineContext extends XMachineContext {
    /**
     * A `ErrorMap` of errors that may have occurred during the machine's workflows.
     */
    readonly errors: ErrorMap<ErrorDetailMap>;
    /**
     * A general machine error. Maybe equal to `errors.lastError`.
     */
    readonly error: any;
}
/**
 * Machine interface.
 *
 * @remarks
 * The interface of our xstate based machines or logic workflows. These contain bindable properties for convenience.
 * Should be extended.
 *
 * @example Creating a machine in the DI.
 * ```ts
 * export interface MyMachine extends Machine<MyMachineContext, MyMachineEvent> {
 *   readonly service: MyService;
 *   ...
 * }
 * export class DefaultMyMachine
 *   extends AbstractMachine<MyMachineContext, MyMachineEvent>
 *   implements MyMachine
 * {
 *   constructor(@MyService readonly service: MyService) {
 *     super();
 *     this.machine = createMachine<MyMachineContext, MyMachineEvent>(...);
 *     this.start(); // < Call now or later to ready the machine.
 *   }
 *   start(
 *     options: InterpreterOptions<typeof this.machine> = {
 *       input: myDefaultContext,
 *     },
 *   ) {
 *     super.start(options);
 *   }
 * }
 * // Whether the machine should be `transient` or a `singleton` probably depends on the use case and if it contains a `final` state.
 * export const MyMachine = DI.createInterface<MyMachine>((x) => x.transient(DefaultMyMachine));
 * ```
 *
 * Machines are bindable from templates and other logic.
 *
 * @example Surface errors in a template.
 * ```ts
 * ${when(
 *   (x) => !!x.myMachine.errors.size,
 *   html<MyComponent>`
 *     <span>${(x) => x.myMachine.errors.messages}</span>
 *   `,
 * )}
 * ```
 *
 * @example Use context data in a class.
 * ```ts
 * this.myMachine.binding((x) => x.context.data, (value) => {
 *   // use data
 * });
 * ```
 *
 * @public
 */
export interface Machine<TContext extends MachineContext, TEvent extends EventObject = AnyEventObject, TAction extends ParameterizedObject = ParameterizedObject, TActor extends ProvidedActor = ProvidedActor, TInput = any, TResolvedTypesMeta = ResolveTypegenMeta<TypegenDisabled, NoInfer<TEvent>, TAction, TActor>> {
    /**
     * Mandatory machine property.
     * @remarks
     * The logic / config that goes into a machine is not accessible from the outside, and is owned by the implementing
     * class. However, consumers of that may provide custom `implementations` in the xstate sense via the `provide()` api.
     */
    readonly machine: StateMachine<TContext, TEvent, TAction, TActor, TInput, TResolvedTypesMeta>;
    /**
     * Optional actor
     * @remarks
     * Callers can decide to reuse/reference if set or create their own.
     */
    readonly actor?: InterpreterFrom<StateMachine<TContext, TEvent, TAction, TActor, TInput, TResolvedTypesMeta>>;
    /**
     * The last `snapshot` from the actor.
     */
    readonly snapshot: State<TContext, TEvent, TActor, TResolvedTypesMeta>;
    /**
     * The current `state` of the machine itself. This is not to be confused with state in the data or store sense.
     */
    readonly state: StateValue;
    /**
     * The `state` of the machine when started. Allows consumers to start a machine from a persisted state.
     */
    readonly startingState: StateValue;
    /**
     * The current `context` of the machine. The data points in context may be considered transient in nature.
     *
     * @privateRemarks
     * Review the possibility of using an object with `@observable` properties as `context`, such as a store fragment or class.
     */
    readonly context: Partial<TContext>;
    /**
     * The `context` of the machine when started.
     */
    readonly startingContext: Partial<TContext>;
    /**
     * The `meta` from the current `state`.
     */
    readonly meta: Record<string, any>;
    /**
     * Indicates if the machine has completed the happy path.
     */
    readonly complete: boolean;
    /**
     * Holds successful machine output if any.
     */
    readonly output: any;
    /**
     * An {@link @genesislcap/foundation-utils#ErrorMap | ErrorMap} of errors that may have occurred during the machine's workflows.
     */
    readonly errors: ErrorMap<ErrorDetailMap>;
    /**
     * A general machine error. Likely equal to `errors.lastError`.
     */
    readonly error: any;
    /**
     * Start the machine.
     *
     * @remarks
     * Called to ready the machine and create the internal actor (interpreter). This is called internally after
     * `createMachine` by convention, however it should also be called after calling {@link Machine.provide | provide}
     * when overriding logic. Calling `start` on an injected machine allows you to restart/reuse it if needed. This is okay
     * for machines registered as `transient` in the DI, but care should be taken when creating a machine registered as a
     * `singleton`. You may need to check the status of the machine and block re-starts.
     *
     * @param options - Optional InterpreterOptions. Can be used to restore to a previously persisted state.
     */
    start(options?: InterpreterOptions<StateMachine<TContext, TEvent, TAction, TActor, TInput, TResolvedTypesMeta>>): any;
    /**
     * Stop the machine.
     */
    stop(): void;
    /**
     * Recreate the internal machine with the provided implementations.
     *
     * @example Using an existing machine with a custom implementation.
     * ```ts
     * @MyMachine readonly machine: MyMachine;
     * ...
     * this.machine.provide({
     *   actors: {
     *     serviceCallX: fromPromise(async ({ input }) => Promise.resolve(new Error('Overridden serviceCallX'))),
     *     onSuccess: (context) => successHandlerInHost(context.data)
     *   },
     *   guards: {
     *     isAssured: ({ context }) => true,
     *   },
     * });
     * // When you change the machine it will stop if running, so you'll have to call {@link Machine.start | start} again.
     * this.machine.start();
     * ```
     *
     * @param implementations - `actions`, `guards`, `actors`... to merge with existing options.
     */
    provide(implementations: InternalMachineImplementations<TContext, TEvent, TAction, TActor, TResolvedTypesMeta, true>): void;
    /**
     * Check `state` matches provided value.
     *
     * @remarks
     * Can be useful if you need to conditionally run logic or render markup.
     *
     * @example
     * ```ts
     * if (machine.matches('inactive')) { ... }
     * ```
     *
     * @param value - A state value.
     */
    matches(value: StateValueFrom<StateMachine<TContext, TEvent, TAction, TActor, TInput, TResolvedTypesMeta>>): boolean;
    /**
     * Check `tags` contain provided tag.
     *
     * @remarks
     * This differs from `matches`, as multiple states may use common tags such as `pending` to indicate some async work
     * is occurring. As with `matches`, `hasTag` can be useful if you need to conditionally run logic or render markup.
     *
     * @example
     * ```ts
     * if (machine.hasTag('pending')) { ... }
     * ```
     *
     * @param tag - A tag.
     */
    hasTag(tag: string): boolean;
    /**
     * Get `meta` values by key.
     *
     * @remarks
     * Meta is cumulative across all state nodes represented by the state value.
     *
     * @param key - A key within the `meta` object. Defaults to `content`.
     */
    getMetaValuesByKey<T = string>(key?: string): T[];
    /**
     * An api to send events to the machine.
     *
     * @param event - An event.
     */
    send(event: TEvent): void;
    /**
     * A custom event handler to send events to the machine.
     *
     * @param event - A custom event.
     */
    onSendEvent(event: CustomEvent<TEvent>): void;
    /**
     * An api to allow the observation of values and arbitrary bindings outside the template engine.
     *
     * @example
     * TS attains the value type information from the machine's interface.
     * ```ts
     * this.myMachine.binding(x => x.context.data, value => {...}); // value is typeof data
     * ```
     *
     * @example
     * You can also use the underlying BindingObserver api should you prefer.
     * ```ts
     * this.myMachine.binding(x => x.error).subscribe({
     *   handleChange(binding, args) {
     *     ...
     *   }
     * })
     * ```
     *
     * @param token - A target lookup token which can take various forms.
     * @param subscriberChangeCallback - {@link SubscriberChangeCallback}.
     * @param isVolatileBinding - Indicates the binding is volatile.
     * @param context - {@link @microsoft/fast-element#ExecutionContext}.
     *
     * @public
     */
    binding<TReturn = any, TParent = any>(token: ((target: this) => TReturn) | keyof this, subscriberChangeCallback?: SubscriberChangeCallback<TReturn> | undefined, isVolatileBinding?: boolean, context?: ExecutionContext): BindingObserver<this, TReturn, TParent>;
    /**
     * Gets the machine from the container.
     *
     * @remarks
     * Calling `getFromContainer` on an injected machine will return a new instance providing it was registered with the
     * DI as `transient`. This is simply a convenience method, and may be useful as some machines contain 'final' states,
     * meaning once they complete they can no longer receive events. However, if you want to continue to work with your
     * injected machine reference, re-calling {@link Machine.start | start} may also be a viable option.
     *
     * Please note that machines are injected into other machines to form parts of complex workflows. In such cases,
     * the parent machine will `invoke` or `spawn` the injected machine's `logic` directly to always create a new actor.
     *
     * @example Invoke an injected machine's `logic` from a parent machine
     * ```ts
     * invoke: {
     *    id: 'dataFetcher',
     *    src: fetchMachine.machine, // < Or `src: 'fetcher'`, and create that actor to interact with the injected machine.
     *    ...
     * }
     * ```
     */
    getFromContainer(): this;
}
/**
 * AbstractMachine base class.
 * @public
 */
export declare abstract class AbstractMachine<TContext extends MachineContext, TEvent extends EventObject = AnyEventObject, TAction extends ParameterizedObject = ParameterizedObject, TActor extends ProvidedActor = ProvidedActor, TInput = any, TResolvedTypesMeta = ResolveTypegenMeta<TypegenDisabled, NoInfer<TEvent>, TAction, TActor>> implements Machine<TContext, TEvent, TAction, TActor, TInput, TResolvedTypesMeta> {
    /**
     * @internal
     */
    protected container: Container;
    /** {@inheritDoc Machine.machine} */
    machine: StateMachine<TContext, TEvent, TAction, TActor, TInput, TResolvedTypesMeta>;
    /** {@inheritDoc Machine.actor} */
    actor: InterpreterFrom<StateMachine<TContext, TEvent, TAction, TActor, TInput, TResolvedTypesMeta>>;
    /** {@inheritDoc Machine.snapshot} */
    snapshot: State<TContext, TEvent, TActor, TResolvedTypesMeta>;
    /** {@inheritDoc Machine.state} */
    state: StateValue;
    /** {@inheritDoc Machine.startingState} */
    startingState: StateValue;
    /** {@inheritDoc Machine.context} */
    context: Partial<TContext>;
    /** {@inheritDoc Machine.startingContext} */
    startingContext: Partial<TContext>;
    /** {@inheritDoc Machine.meta} */
    meta: Record<string, any>;
    /** {@inheritDoc Machine.complete} */
    complete: boolean;
    /** {@inheritDoc Machine.output} */
    output: any;
    /** {@inheritDoc Machine.errors} */
    errors: ErrorMap<ErrorDetailMap>;
    /** {@inheritDoc Machine.error} */
    error: any;
    /**
     * @internal
     */
    private subscription;
    /** {@inheritDoc Machine.start} */
    start(options?: InterpreterOptions<typeof this.machine>): void;
    /** {@inheritDoc Machine.stop} */
    stop(): void;
    /**
     * @internal
     */
    protected interpret(options?: InterpreterOptions<typeof this.machine>): void;
    /**
     * @internal
     */
    protected subscribe(): void;
    /**
     * @internal
     */
    protected applySnapshot(snapshot?: typeof this.snapshot): void;
    /**
     * @internal
     */
    protected unsubscribe(): void;
    /**
     * @internal
     */
    protected startActor(): void;
    /**
     * @internal
     */
    protected stopActor(): void;
    /** {@inheritDoc Machine.provide} */
    provide(implementations: InternalMachineImplementations<TContext, TEvent, TAction, TActor, TResolvedTypesMeta, true>): void;
    /** {@inheritDoc Machine.matches} */
    matches(value: StateValueFrom<typeof this.machine>): boolean;
    /** {@inheritDoc Machine.hasTag} */
    hasTag(tag: string): boolean;
    /** {@inheritDoc Machine.getMetaValuesByKey} */
    getMetaValuesByKey<T = string>(key?: string): T[];
    /** {@inheritDoc Machine.send} */
    send(event: TEvent): void;
    /** {@inheritDoc Machine.onSendEvent} */
    onSendEvent: (event: CustomEvent<TEvent>) => void;
    /** {@inheritDoc Machine.binding} */
    binding<TReturn = any, TParent = any>(token: ((target: this) => TReturn) | keyof this, subscriberChangeCallback?: SubscriberChangeCallback<TReturn> | undefined, isVolatileBinding?: boolean, context?: ExecutionContext): BindingObserver<this, TReturn, TParent>;
    /** {@inheritDoc Machine.getFromContainer} */
    abstract getFromContainer(): this;
}
//# sourceMappingURL=machine.d.ts.map