/// <reference types="webappsec-credential-management" />
import type { DoneInvokeEvent, ErrorEvent } from 'xstate';
import type { MachineContext } from '../../core';
import type { FetchConfig } from './machine';
/**
 * FetchMachineContext interface.
 * @remarks
 * Transient state during the machine's execution.
 * @public
 */
export interface FetchMachineContext extends MachineContext {
    /**
     * The params of the fetch request.
     */
    params: {
        url: string;
        /**
         * See {@link typescript#RequestInit} or {@link typescript#CMRequestInit}.
         */
        requestInit?: RequestInit | CMRequestInit;
        /**
         * Individual fetch requests can provide fetch config overrides when necessary.
         */
        fetchConfig?: FetchConfig;
    };
    /**
     * The abort controller of the fetch request.
     */
    controller: AbortController;
    /**
     * The fetch response.
     */
    response: Response | undefined;
    /**
     * The extracted response data.
     */
    data: any;
}
/**
 * FetchMachineOutput type.
 * @remarks
 * The output of the machine when it reaches it's final state, which is provided to host machines via handlers.
 * @public
 */
export type FetchMachineOutput<TData = any> = Partial<Pick<FetchMachineContext, 'response' | 'error'>> & {
    data: TData;
};
/**
 * FetchMachineEvent type.
 * @remarks
 * The send action events used to invoke the machine.
 * @public
 */
export type FetchMachineEvent = {
    type: 'fetch';
    params: FetchMachineContext['params'];
} | {
    type: 'cancel';
} | DoneInvokeEvent<unknown> | ErrorEvent<unknown>;
/**
 * FetchMachineEventMap type.
 * @remarks
 * The mediation CustomEvent used to invoke machine actions.
 * @public
 */
export type FetchMachineEventMap = {
    'fetch-machine-event': FetchMachineEvent;
};
/**
 * ResponseOkChecker type.
 * @privateRemarks
 * Genesis HTTP-Based APIs may provide unexpected status codes, so special handling can be required.
 * @param response - The {@link typescript#Response}.
 * @returns A boolean to indicate if the response is ok.
 * @public
 */
export type ResponseOkChecker = (response: Response) => boolean;
/**
 * {@inheritDoc ResponseOkChecker}
 * @public
 */
export declare const defaultResponseOkChecker: ResponseOkChecker;
/**
 * ResponseDataExtractor type.
 * @remarks
 * Extracts data from the {@link typescript#Response} and processes it accordingly. The default data extractor will inspect
 * `content-type` and call `JSON.parse()` if it includes `application/json`. Custom data extractors can be used here.
 * @privateRemarks
 * Need to find out if the Genesis http layer returns mixed responses. If needed we can catch and return just the text.
 * @typeParam TData - The expected data type. Defaults to any;
 * @param response - The {@link typescript#Response}.
 * @returns The extracted data.
 * @public
 */
export type ResponseDataExtractor<TData = any> = (response: Response) => Promise<TData>;
/**
 * {@inheritDoc ResponseDataExtractor}
 * @public
 */
export declare const defaultResponseDataExtractor: ResponseDataExtractor;
/**
 * Creates an extractor that applies a deserializer to the extracted response data.
 * @param deserializer - The deserializer to apply.
 * @param extractor - An optional data extractor. Defaults to {@link defaultResponseDataExtractor}.
 * @public
 */
export declare const createResponseDataExtractorWithDeserializer: <TInput = any, TOutput = any>(deserializer: (data: TInput) => Promise<TOutput>, extractor?: ResponseDataExtractor) => ResponseDataExtractor;
/**
 * The default fetch config.
 * @public
 */
export declare const defaultFetchConfig: FetchConfig;
/**
 * Default FetchMachineContext.
 * @public
 */
export declare const defaultFetchMachineContext: Partial<FetchMachineContext>;
//# sourceMappingURL=types.d.ts.map