import { createErrorMap } from '@genesislcap/foundation-utils';
import { assign } from 'xstate';
import { escalate } from 'xstate/actions';
import { logger } from '../utils';
import { isDoneInvokeEvent, isErrorEvent } from './guards';
/**
 * Set error action.
 * @remarks
 * Used to set a top level error on the machine. This may equal to {@link @genesislcap/foundation-utils#ErrorMap} `lastError` in the `errors` property.
 * @privateRemarks
 * Contains done event handling to deal with 200 Nacks.
 * @public
 */
export const setError = assign(({ event }) => {
    if (isErrorEvent(event)) {
        return {
            error: event.data,
        };
    }
    if (isDoneInvokeEvent(event)) {
        logger.debug(`setError action called with non error event type.`, event);
        return {
            error: event.output,
        };
    }
});
/**
 * Set error by key action.
 * @remarks
 * Used to set an error in the {@link @genesislcap/foundation-utils#ErrorMap} by key.
 * @privateRemarks
 * Contains done event handling to deal with 200 Nacks. If done output is an error use it, or fallback to `context.error`
 * and set that under the provided key.
 * @param key - The error key.
 * @public
 */
export const setErrorByKey = (key) => assign(({ context, event }) => {
    var _a;
    let error;
    if (isErrorEvent(event)) {
        error = event.data;
    }
    else if (isDoneInvokeEvent(event)) {
        logger.debug(`setErrorByKey action called with non error event type.`, event);
        error = event.output instanceof Error ? event.output : context.error;
    }
    if (error) {
        const errors = (_a = context.errors) !== null && _a !== void 0 ? _a : createErrorMap(logger.error);
        errors.set(key, error);
        return {
            errors,
        };
    }
});
/**
 * Escalate context error action.
 * @remarks
 * Guard this call with a parent check to avoid warnings when there's no parent.
 * @public
 */
export const escalateError = escalate(({ context }) => context.error);
