import { __awaiter, __decorate, __param } from "tslib";
import { DI, optional } from '@microsoft/fast-foundation';
import { assign, createMachine, fromPromise } from 'xstate';
import { AbstractMachine, createErrorStateNode, isAbortError, isDoneInvokeEvent, isErrorEvent, isSyntaxError, isTypeError, setError, } from '../../core';
import { logger } from '../../utils';
import { APIError, isAPIError, isNetworkError, isNotFoundError, NetworkError } from './errors';
import { defaultFetchConfig, defaultFetchMachineContext } from './types';
/**
 * FetchConfig DI key.
 * @internal
 */
export const FetchConfig = DI.createInterface();
/**
 * Default FetchMachine.
 * @public
 */
let DefaultFetchMachine = class DefaultFetchMachine extends AbstractMachine {
    constructor(config = defaultFetchConfig) {
        super();
        this.config = config;
        this.machine = createMachine({
            id: 'fetchMachine',
            context: ({ input }) => (Object.assign({}, input)),
            initial: 'idle',
            states: {
                idle: {
                    tags: ['idle'],
                    meta: {
                        content: 'Idle.',
                    },
                },
                pending: {
                    tags: ['pending'],
                    meta: {
                        content: 'Fetching, please wait.',
                    },
                    invoke: {
                        src: 'fetch',
                        input: ({ context }) => (Object.assign(Object.assign({}, context.params), { controller: context.controller })),
                        onDone: 'success',
                        onError: [
                            {
                                guard: 'isNetworkError',
                                target: 'error.network',
                            },
                            {
                                guard: 'isNotFoundError',
                                target: 'error.notFound',
                            },
                            {
                                guard: 'isAPIError',
                                target: 'error.api',
                            },
                            {
                                guard: 'isAbortError',
                                target: 'error.abort',
                            },
                            {
                                guard: 'isSyntaxError',
                                target: 'error.syntax',
                            },
                            {
                                target: 'error.unknown',
                            },
                        ],
                    },
                },
                success: {
                    tags: ['success'],
                    meta: {
                        content: 'Success.',
                    },
                    entry: ['setResponse', 'setData', 'successCallback'],
                    /**
                     * Provide final output to parent machines.
                     */
                    type: 'final',
                    output: ({ context }) => ({
                        response: context.response,
                        data: context.data,
                    }),
                },
                error: {
                    tags: ['error'],
                    initial: 'unknown',
                    entry: ['setError', 'errorCallback'],
                    states: {
                        unknown: createErrorStateNode('unknown', 'Unknown error.'),
                        network: createErrorStateNode('network', 'Network error.'),
                        notFound: createErrorStateNode('notFound', 'Not found error.'),
                        api: createErrorStateNode('api', 'API error.'),
                        abort: createErrorStateNode('abort', 'Abort error.'),
                        syntax: createErrorStateNode('syntax', 'Syntax error.'),
                    },
                },
            },
            on: {
                fetch: {
                    actions: ['abortInFlight', 'resetContext', 'setParams'],
                    target: '#fetchMachine.pending',
                },
                cancel: {
                    actions: ['abortInFlight', 'resetContext'],
                    target: '#fetchMachine.idle',
                },
            },
        }, {
            actors: {
                fetch: fromPromise(({ input }) => __awaiter(this, void 0, void 0, function* () {
                    var _a, _b, _c, _d, _e;
                    const { controller, fetchConfig, requestInit, url } = input;
                    const fetcher = (_b = (_a = fetchConfig === null || fetchConfig === void 0 ? void 0 : fetchConfig.fetcher) !== null && _a !== void 0 ? _a : this.config.fetcher) !== null && _b !== void 0 ? _b : fetch;
                    const fetchPolicy = (_c = fetchConfig === null || fetchConfig === void 0 ? void 0 : fetchConfig.fetchPolicy) !== null && _c !== void 0 ? _c : this.config.fetchPolicy;
                    const init = requestInit !== null && requestInit !== void 0 ? requestInit : { method: 'GET' };
                    const okChecker = (_d = fetchConfig === null || fetchConfig === void 0 ? void 0 : fetchConfig.responseOkChecker) !== null && _d !== void 0 ? _d : this.config.responseOkChecker;
                    const dataExtractor = (_e = fetchConfig === null || fetchConfig === void 0 ? void 0 : fetchConfig.responseDataExtractor) !== null && _e !== void 0 ? _e : this.config.responseDataExtractor;
                    return fetchPolicy.execute(() => __awaiter(this, void 0, void 0, function* () {
                        try {
                            const response = yield fetcher(url, Object.assign(Object.assign({}, init), { signal: controller.signal }));
                            const data = yield dataExtractor(response);
                            if (!okChecker(response)) {
                                throw new APIError(response.status, data);
                            }
                            return [data, response];
                        }
                        catch (error) {
                            if (isNetworkError(error)) {
                                throw new NetworkError(url);
                            }
                            throw error; // < API, Syntax / JS Error
                        }
                    }));
                })),
            },
            actions: {
                setParams: assign(({ event }) => {
                    if (event.type === 'fetch') {
                        return {
                            params: event.params,
                            controller: new AbortController(),
                        };
                    }
                }),
                setResponse: assign(({ event }) => {
                    if (isDoneInvokeEvent(event)) {
                        const [_, response] = event.output;
                        return {
                            response,
                        };
                    }
                }),
                setData: assign(({ event }) => {
                    if (isDoneInvokeEvent(event)) {
                        const [data] = event.output;
                        return {
                            data,
                        };
                    }
                }),
                setError,
                abortInFlight: ({ context }) => {
                    if (context.controller) {
                        context.controller.abort();
                    }
                },
                resetContext: assign(() => this.startingContext),
                /**
                 * successCallback.
                 * @remarks
                 * Can be overridden by the caller. Alternatively callers can observe context or successful output.
                 * @public
                 */
                successCallback: () => {
                    logger.debug('FetchMachine.successCallback invoked.');
                },
                /**
                 * errorCallback.
                 * @remarks
                 * Can be overridden by the caller. Alternatively callers can observe context or successful output.
                 * @public
                 */
                errorCallback: () => {
                    logger.debug('FetchMachine.errorCallback invoked.');
                },
            },
            guards: {
                isNetworkError: ({ event }) => (isErrorEvent(event) ? isNetworkError(event.data) : false),
                isAPIError: ({ event }) => (isErrorEvent(event) ? isAPIError(event.data) : false),
                isNotFoundError: ({ event }) => isErrorEvent(event) ? isNotFoundError(event.data) : false,
                isAbortError,
                isSyntaxError,
                isTypeError,
            },
        });
        this.start();
    }
    /** {@inheritDoc Machine.start} */
    start(options = {
        input: defaultFetchMachineContext,
    }) {
        super.start(options);
    }
    /** {@inheritDoc Machine.getFromContainer} */
    getFromContainer() {
        return this.container.get(FetchMachine);
    }
};
DefaultFetchMachine = __decorate([
    __param(0, optional(FetchConfig))
], DefaultFetchMachine);
export { DefaultFetchMachine };
/**
 * FetchMachine DI key.
 * @internal
 */
export const FetchMachine = DI.createInterface((x) => x.transient(DefaultFetchMachine));
